@@ -1,8 +1,8 @@
 # from pkg_resources import get_distribution
 
 # __version__ = get_distribution('scalex').version
-__version__ = '1.0.4'
+__version__ = '1.1.0'
 __author__ = 'Kai Cao'
 __email__ = 'caokai@amss.ac.cn'
 
-from .function import Run, Get_label_Prior, label_reweight, load_file, filter_data, batch_scale
\ No newline at end of file
+from .function import Run, get_prior, label_reweight, load_file, filter_data, batch_scale, TFIDF_LSI
\ No newline at end of file
