@@ -10,6 +10,7 @@ import os
 import scanpy as sc
 from anndata import AnnData
 import scipy
+import sklearn
 import pandas as pd
 from scipy.sparse import issparse
 
@@ -29,7 +30,7 @@ DATA_PATH = os.path.expanduser("~")+'/.uniport/'
 CHUNK_SIZE = 20000
 
 def read_mtx(path):
-    """\
+    """
     Read mtx format data folder including: 
     
         * matrix file: e.g. count.mtx or matrix.mtx or their gz format
@@ -94,7 +95,47 @@ def load_file(path):
         adata.X = scipy.sparse.csr_matrix(adata.X)
 
     return adata
+
+def tfidf(X, n_components, binarize=True, random_state=0):
+    from sklearn.feature_extraction.text import TfidfTransformer
     
+    sc_count = np.copy(X)
+    if binarize:
+        sc_count = np.where(sc_count < 1, sc_count, 1)
+    
+    tfidf = TfidfTransformer(norm='l2', sublinear_tf=True)
+    normed_count = tfidf.fit_transform(sc_count)
+
+    lsi = sklearn.decomposition.TruncatedSVD(n_components=n_components, random_state=random_state)
+    lsi_r = lsi.fit_transform(normed_count)
+    
+    X_lsi = lsi_r[:,1:]
+
+    return X_lsi
+    
+def TFIDF_LSI(adata, n_comps=50, binarize=True, random_state=0):
+    '''
+    Computes LSI based on a TF-IDF transformation of the data from MultiMap. Putative dimensionality 
+    reduction for scATAC-seq data. Adds an ``.obsm['X_lsi']`` field to the object it was ran on. 
+    
+    Input
+    -----
+    adata : ``AnnData``
+        The object to run TFIDF + LSI on. Will use ``.X`` as the input data.
+    n_comps : ``int``
+        The number of components to generate. Default: 50
+    binarize : ``bool``
+        Whether to binarize the data prior to the computation. Often done during scATAC-seq 
+        processing. Default: True
+    random_state : ``int``
+        The seed to use for randon number generation. Default: 0
+    '''
+    
+    #this is just a very basic wrapper for the non-adata function
+    if scipy.sparse.issparse(adata.X):
+        adata.obsm['X_lsi'] = tfidf(adata.X.todense(), n_components=n_comps, binarize=binarize, random_state=random_state)
+    else:
+        adata.obsm['X_lsi'] = tfidf(adata.X, n_components=n_comps, binarize=binarize, random_state=random_state)
 
 def filter_data(
         adata: AnnData,
@@ -114,9 +155,6 @@ def filter_data(
     min_cells
         Filtered out genes that are detected in less than n cells. Default: 0.
         
-    Return
-    ------
-    AnnData
     """
     
 
@@ -126,9 +164,7 @@ def filter_data(
     if log: log.info('Filtering features')
     sc.pp.filter_genes(adata, min_cells=min_cells)
 
-    return adata
-
-def batch_scale(adata, chunk_size=CHUNK_SIZE):
+def batch_scale(adata, use_rep='X', chunk_size=CHUNK_SIZE):
     """
     Batch-specific scale data
     
@@ -136,39 +172,41 @@ def batch_scale(adata, chunk_size=CHUNK_SIZE):
     ----------
     adata
         AnnData
+    use_rep
+        use '.X' or '.obsm'
     chunk_size
         chunk large data into small chunks
     
-    Return
-    ------
-    AnnData
     """
     for b in adata.obs['source'].unique():
         idx = np.where(adata.obs['source']==b)[0]
-        scaler = MaxAbsScaler(copy=False).fit(adata.X[idx])
-        for i in range(len(idx)//chunk_size+1):
-            adata.X[idx[i*chunk_size:(i+1)*chunk_size]] = scaler.transform(adata.X[idx[i*chunk_size:(i+1)*chunk_size]])
+        if use_rep == 'X':
+            scaler = MaxAbsScaler(copy=False).fit(adata.X[idx])
+            for i in range(len(idx)//chunk_size+1):
+                adata.X[idx[i*chunk_size:(i+1)*chunk_size]] = scaler.transform(adata.X[idx[i*chunk_size:(i+1)*chunk_size]])
+        else:
+            scaler = MaxAbsScaler(copy=False).fit(adata.obsm[use_rep][idx])
+            for i in range(len(idx)//chunk_size+1):
+                adata.obsm[use_rep][idx[i*chunk_size:(i+1)*chunk_size]] = scaler.transform(adata.obsm[use_rep][idx[i*chunk_size:(i+1)*chunk_size]])
 
-    return adata
-
-def Get_label_Prior(alpha, celltype1, celltype2):
+def get_prior(celltype1, celltype2, alpha=2):
 
     """
     Create a prior correspondence matrix according to cell labels
     
     Parameters
     ----------
-    alpha
-        the confidence of label, ranges from (1, inf). Higher alpha means better confidence.
     celltype1
         cell labels of dataset X
     celltype2
         cell labels of dataset Y
+    alpha
+        the confidence of label, ranges from (1, inf). Higher alpha means better confidence. Default: 2.0
 
     Return
     ------
-    Couple
-        a prior correspondence matrix 
+    torch.tensor
+        a prior correspondence matrix between cells
     """
 
     Couple = alpha*torch.ones(len(celltype1), len(celltype2))
@@ -185,16 +223,16 @@ def Get_label_Prior(alpha, celltype1, celltype2):
 def label_reweight(celltype):
 
     """
-    Reweight labels to let all cell types share the same total weight 
+    Reweight labels to make all cell types share the same total weight 
     
     Parameters
     ----------
     celltype
-        cell labels 
+        cell labels
 
     Return
     ------
-    Weights
+    torch.tensor
         a vector of weights of cells 
     """
 
@@ -214,47 +252,47 @@ def label_reweight(celltype):
 # @profile
 def Run(
         adatas=None,     
-        adata_cm = None,   
+        adata_cm=None,   
         mode='h',
-        Prior = None,
-        ref_id=None,    
-        save_OT=False,
-        rep_celltype='cell_type',
-        use_specific=True,
         lambda_s=0.5,
-        labmda_recon=1.0,
+        lambda_recon=1.0,
         lambda_kl=0.5,
         lambda_ot=1.0,
+        iteration=30000,
+        ref_id=None,    
+        save_OT=False,
+        use_rep=['X', 'X'],
+        out='latent',
+        label_weight=None,
         reg=0.1,
         reg_m=1.0,
         batch_size=256, 
         lr=2e-4, 
-        max_iteration=30000,
-        loss_type='BCE',
-        seed=124, 
+        enc=None,
         gpu=0, 
+        prior=None,
+        loss_type='BCE',
         outdir='output/', 
-        out='latent',
         input_id=0,
         pred_id=1,
+        seed=124, 
+        num_workers=4,
+        batch_key='domain_id',
+        source_name='source',
+        rep_celltype='cell_type',
         umap=False,
         verbose=False,
         assess=False,
         show=False,
-        source_name='source',
-        batch_key='domain_id',
-        label_weight=None,
-        enc=None,
-        dec=None,
     ):
 
     """
-    main function
+    Run data integration
     
     Parameters
     ----------
     adatas
-        List of AnnData matrices for each dataset.
+        List of AnnData matrices, e.g. [adata1, adata2].
     adata_cm
         AnnData matrices containing common genes.
     mode
@@ -263,24 +301,34 @@ def Run(
         If 'v', integrate data profiled from the same cells (Vertical integration)
         If 'd', inetrgate data without common genes (Diagonal integration)
         Default: 'h'.
-    Prior
-        Prior correspondence matrix. Default: None
-    ref_id
-        Id of reference dataset. Default: None
-    save_OT
-        If True, output a global OT plan. Need more memory. Default: False
-    rep_celltype
-        Names of cell-type annotation in AnnData. Default: 'cell_type'
-    use_specific
-        If True, specific genes in each dataset will be considered. Default: True
     lambda_s
         Balanced parameter for common and specific genes. Default: 0.5
     lambda_recon: 
-        Balanced parameter for reconstruct term. Default: 0.5
+        Balanced parameter for reconstruct term. Default: 1.0
     lambda_kl: 
         Balanced parameter for KL divergence. Default: 0.5
     lambda_ot:
         Balanced parameter for OT. Default: 1.0
+    iteration
+        Max iterations for training. Training one batch_size samples is one iteration. Default: 30000
+    ref_id
+        Id of reference dataset. Default: None
+    save_OT
+        If True, output a global OT plan. Need more memory. Default: False
+    use_rep
+        Use '.X' or '.obsm'. For mode='d' only.
+        If use_rep=['X','X'], use 'adatas[0].X' and 'adatas[1].X' for integration.
+        If use_rep=['X','X_lsi'],  use 'adatas[0].X' and 'adatas[1].obsm['X_lsi']' for integration.
+        If use_rep=['X_pca', 'X_lsi'], use 'adatas[0].obsm['X_pca']' and 'adatas[1].obsm['X_lsi']' for integration.
+        Default: ['X','X']
+    out
+        Output of uniPort. Choose from ['latent', 'project', 'predict'].
+        If out=='latent', train the network and output cell embeddings.
+        If out=='project', project data into the latent space and output cell embeddings. 
+        If out=='predict', project data into the latent space and output cell embeddings through a specified decoder.
+        Default: 'latent'. 
+    label_weight
+        Prior-guided weighted vectors. Default: None
     reg:
         Entropy regularization parameter in OT. Default: 0.1
     reg_m:
@@ -289,24 +337,28 @@ def Run(
         Number of samples per batch to load. Default: 256
     lr
         Learning rate. Default: 2e-4
-    max_iteration
-        Max iterations for training. Training one batch_size samples is one iteration. Default: 30000
-    seed
-        Random seed for torch and numpy. Default: 124
+    enc
+        Structure of encoder
     gpu
         Index of GPU to use if GPU is available. Default: 0
+    prior
+        Prior correspondence matrix. Default: None
+    loss_type
+        type of loss. 'BCE', 'MSE' or 'L1'. Default: 'BCE'
     outdir
         Output directory. Default: 'output/'
-    out
-        Output of uniPort. Choose from ['latent', 'project', 'predict'].
-        If out=='latent', train the network and output cell embeddings.
-        If out=='project', project data into the latent space and output cell embeddings. 
-        If out=='predict', project data into the latent space and output cell embeddings through a specified decoder. 
-        Default: 'latent'.
     input_id
         Only used when mode=='d' and out=='predict' to choose a encoder to project data. Default: 0
     pred_id
         Only used when out=='predict' to choose a decoder to predict data. Default: 1
+    seed
+        Random seed for torch and numpy. Default: 124
+    batch_key
+        Name of batch in AnnData. Default: domain_id
+    source_name
+        Name of source in AnnData. Default: source
+    rep_celltype
+        Names of cell-type annotation in AnnData. Default: 'cell_type'   
     umap
         If True, perform UMAP for visualization. Default: False
     verbose
@@ -315,21 +367,17 @@ def Run(
         If True, calculate the entropy_batch_mixing score and silhouette score to evaluate integration results. Default: False
     show
         If True, show the UMAP visualization of latent space. Default: False
-    source_name
-        Name of source in AnnData. Default: source
-    batch_key
-        Name of batch in AnnData. Default: domain_id
-    label_weight
-        Prior-guided weighted vectors. Default: None
-    enc
-        structure of encoder
-    dec
-        structure of decoder
-    
-    Return
-    ------
-    Weights
-        a vector of weight 
+
+    Returns
+    -------
+    adata.h5ad
+        The AnnData matrice after integration. The representation of the data is stored at adata.obsm['latent'], adata.obsm['project'] or adata.obsm['predict'].
+    checkpoint
+        model.pt contains the variables of the model and config.pt contains the parameters of the model.
+    log.txt
+        Records model parameters.
+    umap.pdf 
+        UMAP plot for visualization if umap=True.
     """
 
     if mode == 'h' and adata_cm is None:
@@ -354,6 +402,8 @@ def Run(
     os.makedirs(outdir+'/checkpoint', exist_ok=True)
     log = create_logger('', fh=outdir+'log.txt')
 
+    use_specific=True
+
     # split adata_cm to adatas
     if adatas is None:  
         use_specific = False
@@ -375,22 +425,29 @@ def Run(
     num_gene = []
 
     for i, adata in enumerate(adatas):
-        print('Dataset {}:'.format(i), adata.obs[source_name][0])
-        print(adata)
-        num_cell.append(adata.X.shape[0])
-        num_gene.append(adata.X.shape[1])
+        if use_rep[i]=='X':
+            num_cell.append(adata.X.shape[0])
+            num_gene.append(adata.X.shape[1])
+        else:
+            num_cell.append(adata.obsm[use_rep[i]].shape[0])
+            num_gene.append(adata.obsm[use_rep[i]].shape[1])
 
-    print('Reference dataset is dataset {}'.format(ref_id))
-    print('\n')
-
-    if adata_cm is not None:
-        print('Data with common HVG')
-        print(adata_cm)
-        print('\n')
 
     # training
     if out == 'latent':
 
+        for i, adata in enumerate(adatas):
+            print('Dataset {}:'.format(i), adata.obs[source_name][0])
+            print(adata)
+
+        print('Reference dataset is dataset {}'.format(ref_id))
+        print('\n')
+
+        if adata_cm is not None:
+            print('Data with common HVG')
+            print(adata_cm)
+            print('\n')
+
         if save_OT:           
             memory = 0
             for i in range(n_domain):
@@ -411,13 +468,15 @@ def Run(
 
         trainloader, testloader = load_data(
             adatas=adatas, 
+            mode=mode,
+            use_rep=use_rep,
             num_cell=num_cell,
             max_gene=max(num_gene), 
             adata_cm=adata_cm,
             use_specific=use_specific, 
             domain_name=batch_key,
-            batch_size=batch_size, 
-            mode=mode
+            batch_size=batch_size,
+            num_workers=4
         )
 
         early_stopping = EarlyStopping(patience=10, checkpoint_file=outdir+'/checkpoint/model.pt')
@@ -427,22 +486,21 @@ def Run(
             enc = [['fc', 1024, 1, 'relu'],['fc', 16, '', '']]      
 
         # decoder structure
-        if dec is None:
-            dec = {} 
-            if mode == 'd':     
-                for i in range(n_domain):          
-                    dec[i] = [['fc', num_gene[i], 1, 'sigmoid']]
+        dec = {} 
+        if mode == 'd':     
+            for i in range(n_domain):          
+                dec[i] = [['fc', num_gene[i], 1, 'sigmoid']]
 
-            elif mode == 'h':      
-                num_gene.append(adata_cm.X.shape[1]) 
-                dec[0] = [['fc', num_gene[n_domain], n_domain, 'sigmoid']]  # common decoder
-                if use_specific: 
-                    for i in range(1, n_domain+1):
-                        dec[i] = [['fc', num_gene[i-1], 1, 'sigmoid']]   # dataset-specific decoder
+        elif mode == 'h':      
+            num_gene.append(adata_cm.X.shape[1]) 
+            dec[0] = [['fc', num_gene[n_domain], n_domain, 'sigmoid']]  # common decoder
+            if use_specific: 
+                for i in range(1, n_domain+1):
+                    dec[i] = [['fc', num_gene[i-1], 1, 'sigmoid']]   # dataset-specific decoder
 
-            else:
-                for i in range(n_domain):
-                    dec[i] = [['fc', num_gene[i], 1, 'sigmoid']]    # dataset-specific decoder
+        else:
+            for i in range(n_domain):
+                dec[i] = [['fc', num_gene[i], 1, 'sigmoid']]    # dataset-specific decoder
 
         # init model
         model = VAE(enc, dec, ref_id=ref_id, n_domain=n_domain, mode=mode)
@@ -456,17 +514,17 @@ def Run(
             num_gene,
             mode=mode,
             label_weight=label_weight,
-            Prior=Prior,
+            Prior=prior,
             save_OT=save_OT,
             use_specific=use_specific,
             lambda_s=lambda_s,
-            labmda_recon=labmda_recon,
+            lambda_recon=lambda_recon,
             lambda_kl=lambda_kl,
             lambda_ot=lambda_ot,
             reg=reg,
             reg_m=reg_m,
             lr=lr, 
-            max_iteration=max_iteration, 
+            max_iteration=iteration, 
             device=device, 
             early_stopping=early_stopping, 
             verbose=verbose,
@@ -519,7 +577,7 @@ def Run(
         # sc.set_figure_params(dpi=200, fontsize=10)
  
         if mode == 'h':
-            sc.pp.neighbors(adata_cm, n_neighbors=30, use_rep=out)
+            sc.pp.neighbors(adata_cm, use_rep=out)
             sc.tl.umap(adata_cm, min_dist=0.1)
             sc.tl.leiden(adata_cm)
             
