@@ -18,7 +18,7 @@ from .loss import *
 
 class VAE(nn.Module):
     """
-    VAE framework
+    Variational Autoencoder framework
     """
     def __init__(self, enc, dec, ref_id, n_domain, mode):
         """
@@ -186,13 +186,14 @@ class VAE(nn.Module):
             num_cell,
             num_gene,
             mode='h',
+            loss_type='BCE',
             label_weight=None,
             Prior=None,
             save_OT=False,
-            use_specific=False,
+            use_specific=True,
             lambda_s=0.5,
             lambda_kl=0.5,
-            labmda_recon=1.0,
+            lambda_recon=1.0,
             lambda_ot=1.0,
             reg=0.1,
             reg_m=1.0,
@@ -201,7 +202,6 @@ class VAE(nn.Module):
             early_stopping=None,
             device='cuda:0',  
             verbose=False,
-            loss_type='BCE'
         ):
         """
         train VAE
@@ -222,6 +222,8 @@ class VAE(nn.Module):
             If 'v', integrate data profiled from the same cells
             If 'd', inetrgate data without common genes
             Default: 'h'
+        loss_type
+            type of loss. Choose between ['BCE', 'MSE'm 'L1']. Default: 'BCE'
         label_weight
             Prior-guided weighted vectors. Default: None
         Prior
@@ -252,11 +254,6 @@ class VAE(nn.Module):
             'cuda' or 'cpu' for training. Default: 'cuda'
         verbose
             Verbosity, True or False. Default: False
-    
-        
-        Returns
-        -------
-   
         """
 
         self.to(device)
@@ -305,7 +302,7 @@ class VAE(nn.Module):
                             recon = self.decoder(z, j)
                             recon_loss += lambda_s * loss_func(recon, x_list[j]) * 2000   ## TO DO
                     
-                        loss = {'recon_loss':labmda_recon*recon_loss, 'kl_loss':lambda_kl*kl_loss} 
+                        loss = {'recon_loss':lambda_recon*recon_loss, 'kl_loss':lambda_kl*kl_loss} 
 
                         optim.zero_grad()
                         sum(loss.values()).backward()
@@ -388,7 +385,7 @@ class VAE(nn.Module):
 
                             ot_loss += ot_loss_tmp
 
-                        loss = {'recon_loss':labmda_recon*recon_loss, 'kl_loss':lambda_kl*kl_loss, 'ot_loss':lambda_ot*ot_loss} 
+                        loss = {'recon_loss':lambda_recon*recon_loss, 'kl_loss':lambda_kl*kl_loss, 'ot_loss':lambda_ot*ot_loss} 
 
                         optim.zero_grad()
                         sum(loss.values()).backward()
@@ -532,7 +529,7 @@ class VAE(nn.Module):
 
                                     ot_loss += ot_loss_tmp
    
-                        loss = {'recloss':labmda_recon*recon_loss, 'klloss':lambda_kl*kl_loss, 'otloss':lambda_ot*ot_loss} 
+                        loss = {'recloss':lambda_recon*recon_loss, 'klloss':lambda_kl*kl_loss, 'otloss':lambda_ot*ot_loss} 
                         
                         optim.zero_grad()
                         sum(loss.values()).backward()
